package samples.catchtherain;

import javafx.animation.AnimationTimer;
import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.input.KeyEvent;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;
import javafx.stage.Stage;

import static java.lang.System.out;

/**
 * The GUI parts and eventhandling of "Catch the Rain" game. Catching raindrops in bucket
 * No application logic here
 *
 * To run the gam run this
 *
 * Note: The coordinate system has y-axis pointing down!
 *
 * Note: There is a known bug in the program (not fixed for pedagogical reasons)
 *
 */
public class CatchTheRainGUI extends Application {


    private final int width = 500;
    private final int height = 500;
    private CatchTheRain catchTR;

    // -- Handle keyboard -----------------------

    private void keyPressed(KeyEvent event) {
        switch (event.getCode()) {
            case LEFT:
                catchTR.bucketLeft();
                break;
            case RIGHT:
                catchTR.bucketRight();
                break;
            default:

        }
    }

    private void keyReleased(KeyEvent event) {
        catchTR.bucketStop();
    }


    @Override
    public void init(){
        int MAX_DROPS = 10;
        Bucket bucket = new Bucket(width / 2, height - 40, 40, 40, Color.BLUE);
        catchTR = new CatchTheRain(bucket, MAX_DROPS, width, height);
    }

    // --------- Methods to render the model ------------------

    private void renderGame(GraphicsContext g) {
        g.clearRect(0, 0, width, height);
        g.fillText("Points: " + catchTR.getPoints(), 10, 20);
        renderRainDrops(g, catchTR.getDrops());  // Order matters, look like drops fall into bucket (not disappear)
        renderBucket(g, catchTR.getBucket());
    }

    // Draw all drops
    private void renderRainDrops(GraphicsContext g, RainDrop[] drops) {
        for (int i = 0; i < drops.length; i++) {
            if (drops[i] != null) {
                renderRainDrop(g, drops[i]);
            }
        }
    }

    // A single drop
    private void renderRainDrop(GraphicsContext g, RainDrop d) {
        g.setFill(d.getColor());
        double size = d.getMaxX() - d.getX();
        g.fillOval(d.getX(), d.getY(), size, size );
    }

    private void renderBucket(GraphicsContext g, Bucket b) {
        g.setFill(b.getColor());
        double w = b.getMaxX() - b.getX();
        double h = b.getMaxY() - b.getY();
        g.fillRect(b.getX(), b.getY(), w, h);
    }

    // --------  JavaFX, setup graphics, event handling and create a timer -----------------------

    @Override
    public void start(Stage primaryStage) throws Exception {

        Group root = new Group();
        Canvas canvas = new Canvas(width, height);
        root.getChildren().addAll(canvas);
        GraphicsContext gc = canvas.getGraphicsContext2D();

        AnimationTimer timer = new AnimationTimer() {
            // Param not used
            public void handle(long currentNanoTime) {
                catchTR.update();
                renderGame(gc);

            }
        };
        Scene scene = new Scene(root);
        scene.setOnKeyPressed(this::keyPressed);
        scene.setOnKeyReleased(this::keyReleased);
        primaryStage.setScene(scene);
        primaryStage.setTitle("Catch the Rain");
        primaryStage.show();
        timer.start();
    }

    public static void main(String[] args) {
        launch(args);
    }
}
