package exercises.fx;

import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

import static javafx.scene.paint.Color.*;


/*
 * Draw the Mandelbrot fractal
 * https://en.wikipedia.org/wiki/Mandelbrot_set
 *
 * See:
 * - mandelbrot.png
 */
public class MandelbrotSet extends Application {

    // Return number of iteration before point diverges
    // Max iterations = 255 (if so point doesn't diverge )
    int getNIterations(double px, double py) {
        // TODO
        return 0;
    }

    Color getColor(int nIterations) {
        if (nIterations < 4) {
            return WHITE;
        } else if (nIterations < 12) {
            return YELLOW;
        } else if (nIterations < 20) {
            return RED;
        } else if (nIterations < 50) {
            return GREEN;
        } else if (nIterations < 100) {
            return MAGENTA;
        } else if (nIterations < 200) {
            return BLUE;
        } else {
            return BLACK;
        }
    }

    public void render(GraphicsContext gc) {
        for (double re = -3; re < 1.5; re = re + 0.005) {
            for (double img = -1.5; img < 1.5; img = img + 0.005) {
                int i = getNIterations(re, img);
                gc.setFill(getColor(i));
                gc.fillOval(scale * re + 2 * width / 3, scale * img + height / 2, 1, 1);
            }
        }
    }

    // ----------- JavaFX ---------------------------------

    final int width = 800;
    final int height = 600;
    final double scale = 200;

    @Override
    public void start(Stage primaryStage) throws Exception {

        Group root = new Group();
        Canvas canvas = new Canvas(width, height);

        root.getChildren().addAll(canvas);
        GraphicsContext gc = canvas.getGraphicsContext2D();

        // Create a scene and connect to the stage
        Scene scene = new Scene(root);
        primaryStage.setScene(scene);
        primaryStage.setTitle("Mandelbrot");
        primaryStage.show();

        render(gc);
    }


}
